package Text::Xslate::EscapedString;
# the package in implemented intentionally in Text::Xslate
use Text::Xslate;
1;
__END__

=head1 NAME

Text::Xslate::EscapedString - The escaped string representation

=cut
